#include "IconInfo.h"

#include <NodeInfo.h>
#include <SymLink.h>
#include <stdio.h>
#include <AppFileInfo.h>

// ICON INFO
IconInfo::IconInfo()
:	valid(true)
{
	args[0] = 0;
}

IconInfo::IconInfo( BMessage * msg )
:	valid(true)
{
	path.SetTo( msg->FindString("path") );
	mime_type = msg->FindString("mime");
	
	SetArgs( msg->FindString("args") );
	
	BEntry entry( path.Path() );
	entry.GetRef( &ref );
}

IconInfo::~IconInfo()
{
}

BArchivable *
IconInfo::Instantiate( BMessage * msg )
{
	if ( validate_instantiation(msg, "IconInfo") ) 
		return new IconInfo(msg);
	return NULL; 
}

status_t
IconInfo::Archive( BMessage * msg, bool deep ) const
{
	BArchivable::Archive(msg, deep);
	
	msg->AddString("path", path.Path() );
	msg->AddString("mime", mime_type.String() );
	msg->AddString("args", args );
	
	return B_OK;
}

const char *
IconInfo::Path() const
{
	return path.Path();
}

entry_ref *
IconInfo::EntryRef()
{
	return &ref;
}

const char *
IconInfo::MimeType() const
{
	return mime_type.String();
}

bool
IconInfo::SetTo( entry_ref e_ref )
{
	ref = e_ref;
	BEntry entry( &ref );
	path.SetTo( &entry );
	
	mime_type = "application/x-vnd.Be-symlink";
	
	BPath real_path(path);
	
	while ( mime_type == "application/x-vnd.Be-symlink" )
	{ // resolve any links to get the 'real' mime type
		BNode node( real_path.Path() );
		BNodeInfo  ninfo( &node );
		char msig[512];
		if ( ninfo.GetType( msig ) == B_OK );
			mime_type = msig;
		
		printf("temp path: [%s] [%s]\n", real_path.Path(), mime_type.String());
		if ( mime_type == "application/x-vnd.Be-symlink" )
		{
			BPath dir;
			real_path.GetParent( &dir );
			
			BEntry t_entry( real_path.Path() );
			BSymLink link( &t_entry );
			if ( link.MakeLinkedPath( dir.Path(), &real_path ) < 0 )
			{ // link resolve fail
				printf("broken link\n");
				valid = false;
				break; // stop searching, broken link
			}
		}
	}
	
	if ( strncmp(mime_type.String(), "application/", 12) == 0 )
	{ // application, set mime_type to signature
		if ( mime_type != "application/x-vnd.Be-symlink" )
		{
			BFile file( path.Path(), B_READ_ONLY );
			BAppFileInfo info( &file );
		
			char mime[B_MIME_TYPE_LENGTH];
			info.GetSignature( mime );
			mime_type = mime;
		}
	}

	return true;
}

const char *
IconInfo::GetArgs()
{
	return args;
}

void
IconInfo::SetArgs( const char * a )
{
	if ( a )
	{
		strcpy(args, a);
	} else
		args[0] = 0;
}

bool
IconInfo::IsValid() const
{
	return valid;
}
